# Copyright (C) 1999, 2000 Jay Beale
# Licensed under the GNU General Public License

package Bastille::DNS;
use lib "/usr/lib";

use Bastille::API;
@ENV="";
$ENV{PATH}="";
$ENV{CDPATH}=".";
$ENV{BASH_ENV}="";

######## TO DO: Convert the chown line to API functions, so we don't
#########       have to use GLOBAL_PREFIX

#######################################################################
##                             DNS/BIND/NAMED                        ##
#######################################################################

&chrootBIND;
&DeactivateNamed;


sub chrootBIND {

    if ( &getGlobalConfig("DNS","chrootbind") eq "Y" ) {
	&ActionLog("# sub chrootBIND\n");

	# Only chroot bind if we are running on a Red Hat / Mandrake system
	# and we haven't already chroot'ed it...
	if($GLOBAL_DISTRO =~ "^HP-UX"){
	    # install generic chroot tree
	    if(&B_install_jail("bind",&getGlobal('BFILE',"jail.generic.hpux"))) {
		# add dev/null and dev/log to chroot tree
		&B_mknod("", &getGlobal('BDIR','jail') . "/bind/dev/null","c 3 0x000002");
		&B_chmod(0666,&getGlobal('BDIR','jail') . "/bind/dev/null");
		&B_System(&getGlobal('BIN','mkfifo') . " -p -m 0666 " . &getGlobal('BDIR','jail') . "/bind/dev/log",
			  &getGlobal('BIN','rm') . " -f " . &getGlobal('BDIR','jail') . "/bind/dev/log");
		# install bind specific files and directories.
		if(&B_install_jail("bind", &getGlobal('BFILE',"jail.bind.hpux"))){
		    &ActionLog("Bind chroot directory setup\n");
		    # changing rc.config.d entry for named so that it will run in the chrooted evironment
		    my $ch_rc = &getGlobal('BIN',"ch_rc");
		    &B_TODO("Chroot'ed Bind:\n" .
			    "Bastille has created a simple named directory structure in\n" .
			    &getGlobal('BDIR',"jail") . "/bind/\n" .
			    "1: Provide all of the configuration files that the current named implementation\n" .
			    "   requires. This is at least a named.conf file. If in the named.conf file under\n" .
			    "   the options section, there is a directory path specified that path must be made\n" .
			    "   as well as all of the files that are referenced from that directory.\n" . 
			    "2: When all of the above is done stop the currently running named process\n" .
			    "   using the systems init script for named and remove all /var/run/named.*\n" .
			    "   files if they exist.\n" .
			    "3: Create the following symlink:\n" . 
			    "   # ln -s " . &getGlobal('BDIR',"jail") . "/bind/var/run/named.pid /var/run/named.pid\n" .
			    "4: Instruct the named init script to chroot into the new environment:\n" .
			    "   # $ch_rc" . ' -a -p NAMED_ARGS="-u daemon -t ' . &getGlobal('BDIR','jail') . '/bind"' . "\n" .
			    "5: Start named in its chroot jail using the named init script\n\n");
			    
		}
	    }
	    
	    
	}
	else{

	    my $home_dns="/home/dns";
	    
	    if (( not ( -d "${GLOBAL_PREFIX}$home_dns" ) ) and (($GLOBAL_DISTRO =~ /^RH/)  or ($GLOBAL_DISTRO =~ /^MN/) )) {
		
		# We place BIND in a chroot'd environment, following the 
		# instructions in SANS Step by Step 4.3.3
		
		# Make the dns chroot directory
		&B_create_dir ( $home_dns );      
		
		# Create the dns user
		&B_append_line ("/etc/passwd",":53:","dns:x:53:53::$home_dns:/bin/false\n");
		&B_append_line ("/etc/group",":53:","dns:x:53:\n");
		&B_append_line("/etc/shadow","^dns:","dns:*:11089:0:99999:7:::\n");
		
		# Populate the dns chroot directory
		&B_create_dir ("$home_dns/etc");
		&B_chmod (0755,"$home_dns/etc");
		&B_create_dir ("$home_dns/lib");
		&B_chmod (0755,"$home_dns/lib");
		&B_create_dir ("$home_dns/dev");
		&B_chmod (0755,"$home_dns/dev");
		&B_create_dir ("$home_dns/usr");
		&B_chmod (0755,"$home_dns/usr");
		&B_create_dir ("$home_dns/usr/sbin");
		&B_chmod (0755,"$home_dns/usr/sbin");
		&B_create_dir ("$home_dns/var");
		&B_chmod (0755,"$home_dns/var");
		&B_create_dir ("$home_dns/var/named");
		&B_chmod (0755,"$home_dns/var/named");
		&B_create_dir ("$home_dns/var/run");
		&B_chmod (0755,"$home_dns/var/run");
		
		unless ($GLOBAL_LOGONLY) {
		    &B_mknod(" -m 666 ","$home_dns/dev/null"," c 1 3");
		    
		    &B_cp("/etc/named.conf","$home_dns/etc/named.conf");
		    
		    # Should we tell them to move their name logs over?
		    
		    &B_cp("/var/named/named.ca","$home_dns/var/named/named.ca");
		    &B_cp("/var/named/named.local","$home_dns/var/named/named.local");
		    
		    my $command=&getGlobal('BIN',"chown");
		    `$command -R dns.dns ${GLOBAL_PREFIX}$home_dns/var/named $home_dns/var/run`;
		    &B_cp("/usr/sbin/named","$home_dns/usr/sbin/named");
		    &B_chmod(0755,"$home_dns/usr/sbin/named");
		    # Don't delete named -- let it get loaded from /usr/sbin, but
		    # find data files in the chroot directory.  We'll still put 
		    # a copy of the binary in the chroot directory so the named
		    # binary can be reloaded from there if necessary.
		    #
		    #&B_delete_file("/usr/sbin/named");
		    &B_cp("/usr/sbin/named-xfer","$home_dns/usr/sbin/named-xfer");
		    &B_chmod(0755,"$home_dns/usr/sbin/named-xfer");
		    &B_cp("/lib/libc.so.6","$home_dns/lib/libc.so.6");
		    &B_cp("/lib/ld-linux.so.2","$home_dns/lib/ld-linux.so.2");
		}
		
		# named communicates normally with syslog via the device /dev/log, 
		# which isn't accessible from the chroot'd environment.  We set 
		# syslog to create and listen to a specific device just for bind.
		
		&B_replace_line("/etc/rc.d/init.d/syslog",'daemon syslogd -m 0\s*\$',"daemon syslogd -m 0 -a $home_dns/dev/log\n");
	
		# Modify named's init script to use the chroot environment
		
		unless ($GLOBAL_DISTRO eq "MN8.0") {
		    &B_replace_line("/etc/rc.d/init.d/named",'^\s*daemon\s+named',"daemon named -u dns -g dns -t $home_dns\n");
		}
		else {
		    # Mandrake 8.0 uses BIND 9, which deprecated the -g option.
		    &B_replace_line("/etc/rc.d/init.d/named",'^\s*daemon\s+named',"daemon named -u dns -t $home_dns\n");  
	      }
		
	    }
	    else {
		# We only do this to Red Hat and Mandrake boxes for now, because
		# we'd have to find all the libraries and modify startup files
		# and this can just be totally difficult to automate when you
		# don't know the target...
		
		&ErrorLog("Couldn't chroot named! Not on a Red Hat / Mandrake box or $home_dns already exists.\n");
	    }
	    
	} 
    }
}


sub DeactivateNamed { 


    if (&getGlobalConfig("DNS","namedoff") eq "Y") {

	&ActionLog("# sub DeactivateNamed\n");

	# Deactivate BIND unless it is being used.

	&B_chkconfig_off ("named");
    }

} 


1;





